
import os
import sys
import time

from PyQt4.QtCore import *
from sface.config import config
from sface.xmlrpcwindow import XmlrpcTracker, XmlrpcReader

def find_executable(exec_name):
    """find the given executable in $PATH"""
    paths = os.getenv("PATH").split(':')
    for p in paths:
        exec_path = os.path.join(p, exec_name)
        if os.path.exists(exec_path):
            return exec_path
    return None


class SfiProcess(QObject):
    def __init__(self, parent=None):
        QObject.__init__(self, parent)

        env = QProcess.systemEnvironment()
        env << "PYTHONPATH=%s" % ":".join(sys.path)
        self.process = QProcess()
        self.process.setEnvironment(env)
        self.connect(self.process, SIGNAL("finished(int, QProcess::ExitStatus)"),
                     self.processFinished)

        self.xmlrpcreader = XmlrpcReader() # this one is for parsing XMLRPC responses
        self.xmlrpctracker = XmlrpcTracker() # this one is for the debug window

        # holds aggregate output from processStandardOutput(); used by xmlrpc
        # tracker.
        self.output = ""

        self.connect(self.process, SIGNAL("readyReadStandardOutput()"),
                     self.processStandardOutput)
        self.connect(self.process, SIGNAL("readyReadStandardError()"),
                     self.processStandardError)

    def __init_command(self, args):
        self.args = QStringList()
        self.args << "-d"
        self.args << config.get_dirname()

        # this shows xmlrpc conversation, see sfi.py docs.
        # always do this, so we can parse the XML result for faults and show
        # then to the users.
        self.args << QString('-D')

        for arg in args:
            self.args << QString(arg)

        self.exe = find_executable("sfi.py")
        if not self.exe:
            print "FATAL.. Could not locate binary sfi.py - not much we can do without that"

    def isRunning(self):
        return self.process.state() != QProcess.NotRunning

    def processStandardOutput(self):
        output = self.process.readAllStandardOutput()
        self.output = self.output + output
        if config.debug:
            print output

    def processStandardError(self):
        print self.process.readAllStandardError()

    def processFinished(self):
        if self.process.exitStatus() == QProcess.CrashExit:
            print self.readOutput()
            print "Process exited with errors:",
            err = self.process.error()
            if err == QProcess.FailedToStart:
                print "FailedToStart"
            elif err == QProcess.Crashed:
                print "Crashed"
            elif err == QProcess.Timedout:
                print "Timedout"
            elif err == QProcess.WriteError:
                print "WriteError"
            elif err == QProcess.ReadError:
                print "ReadError"
            elif err == QProcess.UnknownError:
                print "UnknownError"

        # extract any faults from the XMLRPC response(s)
        self.xmlrpcreader.responses = []
        self.xmlrpcreader.store(self.output)
        self.xmlrpcreader.extractXml()
        self.responses = self.xmlrpcreader.responses
        self.faults = [x for x in self.responses if (x["kind"]=="fault")]

        self.trace_end()
        self.emit(SIGNAL("finished()"))

    def getFaultString(self):
        if self.faults == []:
            return None

        return self.faults[0].get("faultString","") + " (" + self.faults[0].get("faultCode","") + ")"

    def retrieveRspec(self):
        slice = config.getSlice()
        mgr = config.getSlicemgr()
        # Write RSpec to file
        filename = config.fullpath ("%s.rspec"%slice)
        try:
            os.remove(filename)
        except:
            pass
        args = ["-u", config.getUser(), "-a", config.getAuthority(), 
                "-r", config.getRegistry(), "-s", mgr, "resources", 
                "-o", filename, slice]

        self.__init_command(args)
        self.start()
        return filename


    def listRecords(self, hrn, rectype=None, filename=None):
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "list", hrn]

        if filename:
            # we can't tell whether SFI will create one file or many, so delete
            # leftovers from last time, then we'll know what we got, after we get it.
            if os.path.exists(filename):
                os.remove(filename)
            if os.path.exists(filename + ".1"):
                os.remove(filename + ".1")
            args.append("-o")
            args.append(filename)

        if rectype:
            args.append("-t")
            args.append(rectype)

        self.__init_command(args)
        self.start()

    def getRecord(self, hrn, filename=None):
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "show", hrn]
        if filename:
            args.append("-o")
            args.append(filename)
        self.__init_command(args)
        self.start()

    def getSliceRecord(self):
        self.getRecord(config.getSlice(), config.getSliceRecordFile())

    def getAuthorityRecord(self):
        self.getRecord(config.getAuthority(), config.getAuthorityRecordFile())

    def applyRSpec(self, rspec):
        # that's what we pass, like in what we'd like to get
        requested = config.getSliceRSpecFile() + "_new"
        # that's what we actually receive
        # just overwrite the slice file as if we'd used 'resources'
        obtained = config.getSliceRSpecFile()
        rspec.save(requested)
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "create",
                "-o", obtained,
                config.getSlice(), requested]
        self.__init_command(args)
        self.start()

    def updateRecord(self, filename):
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "update", filename]
        self.__init_command(args)
        self.start()

    def addRecord(self, filename):
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "add", filename]
        self.__init_command(args)
        self.start()

    def renewSlivers(self, expiration):
        args = ["-u", config.getUser(), "-a", config.getAuthority(),
                "-r", config.getRegistry(), "-s", config.getSlicemgr(), "renew",
                config.getSlice(), expiration]
        self.__init_command(args)
        self.start()

    def start(self):
        self.respones = []
        self.faults = []
        self.output = ""
        self.trace_command()
        self.process.start(self.exe, self.args)

    def readOutput(self):
        if self.process.state() == QProcess.NotRunning:
            return self.process.readAll()

    def trace_command (self):
        if config.verbose:
            self._trace=time.time()
            command = "%s %s" % (self.exe, self.args.join(" "))
            print time.strftime('%H:%M:%S'),'Invoking',command

    def trace_end (self):
        if config.verbose:
#            command = "%s %s" % (self.exe, self.args.join(" "))
            print time.strftime('%H:%M:%S'),"Done [%.3f s]"%(time.time()-self._trace)
        if config.debug:
            self.xmlrpctracker.getAndPrint(self.output)

